#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase, main
from excalibur.loader import ConfigurationLoader
from excalibur.core import PluginsRunner
from excalibur.core import Query
from excalibur.exceptions import PluginRunnerError
from excalibur.exceptions import IPNotAuthorizedError,WrongSignatureError
from excalibur.utils import ALL_KEYWORD
import yaml

class PluginsTest(TestCase):

    def setUp(self):
        # Params of the deactivate method for uds
        self.query = Query(source="etab1",
                           remote_ip="127.0.0.1",
                           signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                           arguments={"login": "testzombie1", },
                           ressource="actions",
                           method="action1",
                           request_method="GET"
                           )

        self.query2 = Query(source="etab1",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action2",
                            request_method="GET"
                            )

        # Files
        self.plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")

        self.data_ok = {'Plugin1': 'p1ok1', 'Plugin2': 'p2ok1'}
        self.errors_raw = {'Plugin1':
                           {'error_message': 'error plugin 1 action 2 !',
                            'method': 'action2',
                            'source': 'etab1',
                            'arguments': {'login': 'testzombie1'},
                            'error': 'Exception',
                            'ressource': 'actions',
                            'parameters_index': 0},
                           'Plugin2':
                          {'error_message': 'error plugin 2 action 2 !',
                           'method': 'action2',
                           'source': 'etab1',
                           'arguments': {'login': 'testzombie1'},
                           'error': 'Exception', 'ressource':
                           'actions',
                           'parameters_index': 0}
                           }

    def test_run_plugins(self):
        data, errors = self.plugin_runner.run_plugins(self.query)

        self.assertEqual(data, self.data_ok)
        self.assertEqual(errors, {})

    def test_run_plugins_error(self):
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner.run_plugins(self.query2)

        self.assertEqual(errors, self.errors_raw)
        self.assertEqual(data, {})

    def test_keyerror_plugins(self):
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sourcesnoplugins.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        with self.assertRaises(PluginRunnerError):
            plugin_runner.plugins(self.query.source, self.query.project)

    def test_plugins_module(self):
        self.assertEqual(self.plugin_runner.plugins_module, "tests.plugins")

    def test_query(self):
        self.assertEqual(self.query.__str__(),
                         "project:None,source:etab1,ip:127.0.0.1,\
sign:c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10,args:{'login': 'testzombie1'},\
ressource:actions,method:action1, request_method:GET")


class RunnerTest(TestCase):

    def setUp(self):

        self.query = Query(source="etab1",
                           remote_ip="127.0.0.1",
                           signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                           arguments={"login": "testzombie1", },
                           ressource="actions",
                           method="action1",
                           request_method="GET"
                           )

        self.query2 = Query(source="etab1",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action2",
                            request_method="GET"
                            )

        self.plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")

        self.data_ok = {'Plugin1': 'p1ok1', 'Plugin2': 'p2ok1'}
        self.errors_raw = {'Plugin1':
                           {'method': 'action2',
                            'source': 'etab1',
                            'arguments': {'login': 'testzombie1'},
                            'error': 'Exception',
                            'error_message': 'error plugin 1 action 2 !',
                            'ressource': 'actions',
                            'parameters_index': 0},
                           'Plugin2':
                           {'method': 'action2',
                            'source': 'etab1',
                            'arguments': {'login': 'testzombie1'},
                            'error': 'Exception',
                            'error_message': 'error plugin 2 action 2 !',
                            'ressource': 'actions',
                            'parameters_index': 0}
                           }
        self.raw_acl = """
etab1:
    actions:
        - action1
        - action2

etab2:
    actions:
        - action1

"""
        self.raw_sources = """
etab1:
    apikey: S3CR3T
    ip:
            - 127.0.0.1
    plugins:

        Plugin1:

            -   spore: S3CR3T
                token: S3CR3T
        Plugin2:
            -   spore: S3CR3T
                token: S3CR3T

        Plugin3:
            -   spore: S3CR3T
                token: S3CR3T

etab2:
    apikey: S3CR3T2
    ip:
            - 127.0.0.1
    plugins:

        Plugin1:

            -   spore: S3CR3T2
                token: S3CR3T2
        Plugin2:
            -   spore: S3CR3T2
                token: S3CR3T2
"""
        self.raw_ressources ="""
actions:
    action1:
        request method: GET
        arguments:
            login:
                checks:
                    min length: 2
                    max length: 50

    action2:
        request method: GET
        arguments:
            login:
                checks:
                    min length: 2
                    max length: 50
"""


    def test_runner(self):
        
        data, errors = self.plugin_runner(self.query)
        self.assertEqual(data, self.data_ok)
        self.assertEqual(errors, {})

    def test_runner_errors(self):
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner(self.query2)
        self.assertEqual(errors, self.errors_raw)
        self.assertEqual(data, {})

    def test_sources_names(self):
        self.assertEqual(
            self.plugin_runner.sources_names(), ['etab1', 'etab2'])

    def test_runner_with_raw_yaml(self):
      plugin_runner = PluginsRunner(
          self.raw_acl,
          self.raw_sources,
          self.raw_ressources,
          "tests.plugins",
          raw_yaml_content=True)

      data, errors = plugin_runner(self.query)
      self.assertEqual(data, self.data_ok)
      self.assertEqual(errors, {})

    def test_runner_with_raw_yaml_errors(self):
      plugin_runner = PluginsRunner(
          self.raw_acl,
          self.raw_sources,
          self.raw_ressources,
          "tests.plugins",
          raw_yaml_content=True)

      data, errors = plugin_runner(self.query2)
      self.assertEqual(errors, self.errors_raw)
      self.assertEqual(data, {})


class RunnerWithProjectsTest(TestCase):

    def setUp(self):

        self.query = Query(source="etab1",
                           remote_ip="127.0.0.1",
                           signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                           arguments={"login": "testzombie1", },
                           ressource="actions",
                           method="action1",
                           request_method="GET",
                           project="project1"
                           )

        self.query2 = Query(source="etab1",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action2",
                            request_method="GET",
                            project="project1"
                            )
        
        self.query3 = Query(source=ALL_KEYWORD,
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action1",
                            request_method="GET",
                            project="project1"
                            )
        
        self.query4 = Query(source="etab1,etab2",
                            remote_ip="127.0.0.1",
                            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                            arguments={"login": "testzombie1", },
                            ressource="actions",
                            method="action1",
                            request_method="GET",
                            project="project1"
                            )
         
        self.plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sources_projects.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        
        self.plugin_runner2 = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/query4_project.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        
        self.data_ok = {'Plugin1': 'p1ok1', 'Plugin2': 'p2ok1'}
        self.errors_raw = {'Plugin1':
                           {'method': 'action2',
                            'source': 'etab1',
                            'arguments': {'login': 'testzombie1'},
                            'error': 'Exception',
                            'error_message': 'error plugin 1 action 2 !',
                            'ressource': 'actions',
                            'parameters_index': 0},
                           'Plugin2':
                           {'method': 'action2',
                            'source': 'etab1',
                            'arguments': {'login': 'testzombie1'},
                            'error': 'Exception',
                            'error_message': 'error plugin 2 action 2 !',
                            'ressource': 'actions',
                            'parameters_index': 0}}

        self.raw_acl = """
project1:
    etab1:
        actions:
            - action1
            - action2

    etab2:
        actions:
            - action1

project2:
    etab1:
        actions:
            - action1
            - action2

"""
        self.raw_sources = """
project1:
     sources:
        etab1:
            apikey: S3CR3T
            ip:
                    - 127.0.0.1
            plugins:

                Plugin1:

                    -   spore: S3CR3T
                        token: S3CR3T
                Plugin2:
                    -   spore: S3CR3T
                        token: S3CR3T

        etab2:
            apikey: S3CR3T2
            ip:
                    - 127.0.0.1
            plugins:

                Plugin1:

                    -   spore: S3CR3T2
                        token: S3CR3T2
                Plugin2:
                    -   spore: S3CR3T2
                        token: S3CR3T2


project2:
    sources:
        etab1:
            apikey: S3CR3T
            ip:
                    - 127.0.0.1
            plugins:

                Plugin1:

                    -   spore: S3CR3T
                        token: S3CR3T
                Plugin2:
                    -   spore: S3CR3T
                        token: S3CR3T
"""
        self.raw_ressources ="""
actions:
    action1:
        request method: GET
        arguments:
            login:
                checks:
                    min length: 2
                    max length: 50

    action2:
        request method: GET
        arguments:
            login:
                checks:
                    min length: 2
                    max length: 50
"""

    def test_runner(self):
        data, errors = self.plugin_runner(self.query)
        self.assertEqual(data, self.data_ok)
        self.assertEqual(errors, {})
        
    def test_runner_query4_one_requested_source_does_not_match(self):
        with self.assertRaises(WrongSignatureError):
            data, errors = self.plugin_runner(self.query4)
            
    def test_runner_query4_one_requested_source(self):
        """
        """
        data, errors = self.plugin_runner2(self.query4)
        self.assertTrue(data["etab2|Plugin2"]=="p2ok1" and data["etab1|Plugin1"]=="p1ok1" and data["etab1|Plugin2"]=='p2ok1')

    def test_runner_errors(self):

        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sources_projects_encoded_api_keys.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner(self.query2)
        self.assertEqual(errors, self.errors_raw)
        self.assertEqual(data, {})
        
    def test_runner_with_query_source_set_to_all(self):
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sources_projects_encoded_api_keys.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner(self.query3)
        self.assertTrue(data["etab1|Plugin1"]=="p1ok1" and data["etab1|Plugin2"]=='p2ok1')
        self.assertTrue("etab2|Plugin2" not in data)
        
    def test_query_source_set_to_all_and_multiple_etabs(self):
        """
        Checks that with 'source==all', data are correctly returned from
        multiple establishments 
        """
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/source_set_to_all_and_multiple_etabs.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner(self.query3)
        self.assertTrue(data["etab2|Plugin2"]=="p2ok1" and data["etab1|Plugin1"]=="p1ok1")
        
    def test_request_set_to_all_and_checksignature_set_to_false(self):
        """
        """
        plugin_runner = PluginsRunner(
            "./tests/data/acl_three_etabs.yml",
            "./tests/data/source_set_to_all_multiple_etabs_one_missing_api_key.yml",
            "./tests/data/ressources.yml",
            "tests.plugins",
            check_signature=False
            )
        data, errors = plugin_runner(self.query3)
        self.assertTrue(data["etab2|Plugin2"]=="p2ok1" and data["etab1|Plugin1"]=="p1ok1")
        
    
        
    def test_query_source_set_to_all__multiple_etabs_multiple_ips(self):
        """
        
        """
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sources_many_ip.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner(self.query3)
        self.assertTrue(data["etab2|Plugin2"]=="p2ok1" and data["etab1|Plugin1"]=="p1ok1")
        self.assertTrue("etab3|Plugin2" not in data)
        
    def test_query_all_missing_ip_in_an_etab(self):
        """
        
        """
       
        
        plugin_runner = PluginsRunner(
        "./tests/data/acl_projects.yml",
        "./tests/data/sources_all_with_apikey_matching_but_ip_not_authorized.yml",
        "./tests/data/ressources.yml",
        "tests.plugins")

       
        with self.assertRaises(IPNotAuthorizedError):
            data, errors = plugin_runner(self.query3)
            
#         self.assertTrue(data["Plugin2"]=="p2ok1" and data["Plugin1"]=="p1ok1")
        
    def test_query_source_set_to_all_and_multiple_etabs_with_errors(self):
        """
        Checks that with 'source==all', but ip does not match
        errors are raised
        NEEDS IP_CHECK TO BE REENABLED
        """
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/source_set_to_all_and_multiple_etabs.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        data, errors = plugin_runner(self.query3)
     
#         self.assertTrue(data["Plugin2"]=="p2ok1" and data["Plugin1"]=="p1ok1")
        

    def test_sources_names(self):
        self.assertEqual(self.plugin_runner.sources_names(self.query.project),
                         ['etab1', 'etab2'])
        self.assertEqual(self.plugin_runner.sources_names("project2"),
                         ['etab1'])
        
    def test_failing_sources_names(self):
        p = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sourcesnoplugins.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        with self.assertRaises(PluginRunnerError):
            p.sources_names(self.query.project)

    def test_runner_with_raw_yaml(self):
      plugin_runner = PluginsRunner(
          self.raw_acl,
          self.raw_sources,
          self.raw_ressources,
          "tests.plugins",
          raw_yaml_content=True)

      data, errors = plugin_runner(self.query)
      self.assertEqual(data, self.data_ok)
      self.assertEqual(errors, {})

    def test_runner_with_raw_yaml_errors(self):
      plugin_runner = PluginsRunner(
          self.raw_acl,
          self.raw_sources,
          self.raw_ressources,
          "tests.plugins",
          raw_yaml_content=True)

      data, errors = plugin_runner(self.query2)
      self.assertEqual(errors, self.errors_raw)
      self.assertEqual(data, {})
      
   
        
if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Class to load configuration and plugins
"""

from excalibur.exceptions import ExcaliburError, ConfigurationLoaderError,\
    PluginLoaderError
import importlib
import yaml


class ConfigurationLoader(object):

    """
    load config from yaml
    """

    def __init__(self, myyaml, raw_yaml_content=False, key=None):
        """
        Load yaml file, or raw yaml if raw_yaml_content is True
        """
        prefix = ""
        if key:
            prefix = """!!python/object:excalibur.conf.""" + key.title() + """
"""
        try:
            if not raw_yaml_content:
                with open(myyaml, "r") as configuration_file:
                    self.__content = yaml.load(
                        prefix + configuration_file.read())
            else:
                self.__content = yaml.load(prefix + myyaml)
        except Exception as e:
            raise ConfigurationLoaderError(
                "error with the configuration loader: %s" % myyaml)

    @property
    def content(self):
        """
        Return configuration content.
        """
        return self.__content


class PluginLoader(object):

    """
    Create plugin instance
    """

    def __init__(self, plugin_module):
        self.plugin_module = plugin_module

    def get_plugin(self, plugin_name):
        """
        return plugin instance
        """
        try:
            module = importlib.import_module(
                "%s.%s" % (self.plugin_module, plugin_name))
            plugin = getattr(module, plugin_name)
            return plugin()
        except Exception as e:
            raise PluginLoaderError(
                "Plugin %s failed to load: %s" % (plugin_name, e))



# -*- coding: utf-8 -*-
import re

from excalibur.exceptions import ArgumentError,\
    ArgumentCheckMethodNotFoundError, CheckMethodError,\
    NoACLMatchedError, RessourceNotFoundError, MethodNotFoundError,\
    HTTPMethodError, SourceNotFoundError, \
    IPNotAuthorizedError, WrongSignatureError
from excalibur.decode import DecodeArguments
from excalibur.utils import add_args_then_encode,\
    ALL_KEYWORD, SOURCE_SEPARATOR, sources_list_or_list,\
    all_sources_or_sources_list_or_list,\
    is_simple_request_and_source_not_found, ip_found_in_sources,\
    get_api_keys_by_sources

import itertools


class Check(object):

    """
    Check parent class.
    Mostly empty.
    """

    def check(self):
        raise NotImplementedError


class CheckArguments(Check):

    """
    Class ensuring argument consistency.
    Search the ressources.yml's argument entry
    to find all registered constraints and launch
    matching methods based on naming convention.
    By now, checks are two arguments methods that
    compare a received value to an expected value.
    The comparator is not dynamically obtained, it
    is in the body of the method.
    """

    @DecodeArguments
    def __init__(self, query, ressources, sources, acl,
                 sha1check=True, ipcheck=True):

        self.ressources = ressources
        self.arguments = query.arguments
        self.ressource = query.ressource
        self.method = query.method
        self.query = query

    def __call__(self):

        # Keep trace of arguments that do not pass tests.
        errors = {}
        targeted_ressource = self.ressources[self.ressource]\
            if self.ressource in self.ressources.keys() else None

        if targeted_ressource is None:
            raise ArgumentError("unexpected argument")

        if "arguments" in targeted_ressource[self.method].keys():

            args = targeted_ressource[self.method][
                "arguments"]
            for argument_name in self.arguments:
                try:
                    check_list = args[argument_name]["checks"]\
                        if args is not None else []
                except KeyError as k:
                    raise ArgumentError("unexpected argument %s"
                                        % argument_name)
                for check in check_list:
                    try:
                        check_method_name = self.format(check)
                        check_method = getattr(self, check_method_name)
                        check_parameter = args[argument_name]["checks"][check]
                        value_to_check = self.arguments[argument_name]
                        if not check_method(value_to_check, check_parameter):
                            errors[argument_name] = check
                    except AttributeError as a:
                        raise ArgumentCheckMethodNotFoundError(
                            check_method_name)
                    except Exception as e:
                        raise CheckMethodError(e)

        if errors:
            raise ArgumentError("The check list did not pass", errors)

    def check_min_length(self, argument_value, length):
        return len(argument_value) >= length

    def check_max_length(self, argument_value, length):
        return len(argument_value) <= length

    def check_value_in(self, argument_value, choices):
        return argument_value in choices

    def check_matches_re(self, argument_value, re_string):
        regex = re.compile(re_string)

        return regex.match(argument_value) is not None

    @staticmethod
    def format(x):
        return "check_" + x.replace(" ", "_")


class CheckACL(Check):

    """
    Checks method and ressources allowances. ACLs are defined
    in the acl.yml file.
    Right now, if we want to be enable requests that parse all sources,
    acl verification MUST be disabled when the source does not match any entry
    in the acl file.
    Yet, checking methods availability is a good principle.
    An equivalent of this check could happen when the method in the core
    is about to be called.
    """

    def __init__(self, query, ressources, sources, acl,
                 sha1check=True, ipcheck=True):
        self.sources = sources
        self.acl = acl
        self.source = query.source
        self.ressource = query.ressource
        self.method = query.method
        self.project = query.project

    def __call__(self):
        targets = all_sources_or_sources_list_or_list(
            self.source, self.sources)

        allowed_method_suffixes = []
        try:
            for target in targets:
                allowed_method_suffixes +=\
                    self.acl[self.project][target][self.ressource]\
                    if self.project else self.acl[target][self.ressource]

            if self.method not in allowed_method_suffixes:
                raise NoACLMatchedError(
                    "%s/%s" % (self.ressource, self.method))
        except KeyError as k:
            raise NoACLMatchedError(
                "%s/%s" % (self.ressource, self.method))


class CheckRequest(Check):

    """
    Check request legitimacy based on various
    criteria registered in the ressources.yml file.
    """

    def __init__(self, query, ressources, sources, acl,
                 sha1check=True, ipcheck=True):

        self.ressources = ressources
        self.http_method = query.request_method
        self.method = query.method
        self.arguments = query.arguments
        self.ressource = query.ressource

    def __call__(self):
        try:
            # sure??
            if self.ressource not in self.ressources.keys():
                raise RessourceNotFoundError(self.ressource)

            if self.method not in self.ressources[self.ressource]:
                raise MethodNotFoundError(self.method)

            targeted_method = self.ressources[self.ressource][self.method]
            if "request method" in\
                targeted_method.keys()\
                and self.http_method !=\
                    targeted_method["request method"]:
                raise HTTPMethodError(
                    targeted_method["request method"])

            if "arguments" in targeted_method.keys():
                # method requires strictly no arguments and received arguments
                # contained parameters
                if not targeted_method['arguments']\
                        and self.arguments != {}:
                    raise ArgumentError("%s only supports no arguments "
                                        "requests, received : %s" % (
                                            self.method, self.arguments))

                # wrong format in arguments received from the request
                if not isinstance(self.arguments, dict):
                    raise ArgumentError("%s is not a supported format" % (
                        targeted_method))

                # required args are missing
                if not self.all_required_args_found(
                        targeted_method["arguments"]):
                    raise ArgumentError("received arguments do no match :"
                                        " %s required arguments : %s)" % (
                                            self.arguments,
                                            targeted_method["arguments"]))

                # args found neither in required nor optional args found
                if not set(self.arguments.keys()).issubset(
                        set(targeted_method['arguments'].keys()if
                            targeted_method['arguments']is not None else[])):
                    raise ArgumentError("exceeding parameters")

        except KeyError as k:
            raise ArgumentError("key not found in sources")

    def all_required_args_found(self, required):
        received = self.required_received_params(required).keys()
        expected = self.required_params(required).keys()
        return set(received) == set(expected)

    def required_params(self, args):

        return {k: v for k, v in args.items() if "optional" not in v.keys()
                or v['optional']is not True}\
            if args is not None else{}

    def required_received_params(self, args):
        return {k: v for k, v in self.arguments.items() if k in
                self.required_params(args).keys()}


class CheckSource(Check):
    """
    Check source ensures that the right api_key is found
    in the plugin's configuration which is targeted by the request.
    It is also the application spot where the ip can be checked.
    """

    def __init__(self, query, ressources, sources, acl,
                 sha1check=True, ipcheck=True):

        self.sources = sources
        self.source = query.source
        self.ip = query.remote_ip
        self.signature = query.signature
        self.arguments = query.arguments
        self.sha1check = sha1check
        self.ipcheck = ipcheck

        self.disable_check("apikey", "sha1check")
        self.disable_check("ip", "ipcheck")

    def disable_check(self, key_name, property_name):
        # for yaml loaded classes to work, disable check should be modified
        if getattr(self, property_name) and \
           self.source in self.sources.keys() and \
           key_name not in self.sources[self.source]:
            setattr(self, property_name, False)

    def __call__(self):
        """

        """

        try:
            if is_simple_request_and_source_not_found(self.source,
                                                      self.sources):
                raise SourceNotFoundError("Unknown source %s" % self.source)
            if self.ipcheck:
                # Check if IP is authorized
                ip_authorized = ip_found_in_sources(self.source, self.sources, self.ip)
                if not ip_authorized:
                    raise IPNotAuthorizedError(self.ip)
            # Signature check
            if self.source != ALL_KEYWORD and self.sha1check:
                # The request has to be allowed for all the sources it targets
                targets = sources_list_or_list(self.source)
                api_keys_by_sources = get_api_keys_by_sources(self.sources,
                                                              targets)
                arguments_list = sorted(self.arguments)

                # If multiple api_keys are registered
                for target_name, api_keys in api_keys_by_sources.items():
                    if len(api_keys) > 1:
                        signkeys = [add_args_then_encode(signature,
                                                         arguments_list,
                                                         self.arguments)
                                    for signature in api_keys]
                        if self.signature not in signkeys:
                            raise WrongSignatureError(self.signature)
                    # If there is only one api_key
                    else:
                        signkey = add_args_then_encode(api_keys[0],
                                                       arguments_list,
                                                       self.arguments)
                        if self.signature != signkey:
                            raise WrongSignatureError(self.signature)
        except KeyError as k:
            raise SourceNotFoundError("key was not found in sources")
        except TypeError as t:
            raise SourceNotFoundError("key was not found in sources")



# -*- coding: utf-8 -*-
"""
Module qui permet de gerer l'encodage de certains arguments. Comme par exemple,
l'encodage en base64 pour palier au probleme de
caracteres speciaux non geres par
les urls.
"""
from excalibur.exceptions import ArgumentError, DecodeAlgorithmNotFoundError
import base64


class DecodeArguments(object):

    """
    Classe contenant les differents algorithmes de decodage.

    Pour implementer un nouvel algorithme, une methode suivant la
    regle de nommage suivante decode_nomalgo doit recevoir la
    valeur en argument. Et retourner la valeur decodee.
    """

    def __init__(self, f):
        self.f = f

    def __get__(self, instance, owner):
        self.cls = owner
        self.obj = instance
        return self.__call__

    def __call__(self, *k, **kw):
        """
        Pour chacun des arguments passes, regarde s'il doit etre decode.
        Et si c'est le cas, appelle la methode correspondante.
        """
        self.ressources = k[1]
        self.ressource = k[0].ressource
        self.method_name = k[0].method
        self.arguments = k[0].arguments

        if self.ressource not in self.ressources.keys():
            raise ArgumentError("ressource not found")

        ressource = self.ressources[self.ressource]

        # the check is optionnal, it occurs only if there is an entry
        # "arguments
        if "arguments" in ressource[self.method_name].keys():
            try:
                arguments = ressource[self.method_name]["arguments"]
                if arguments:
                    for argument_name in self.arguments:
                        if "encoding" in arguments[argument_name]:
                            algo = arguments[argument_name]["encoding"]
                            method = getattr(self, "decode_" + algo)
                            self.arguments[argument_name] = method(
                                self.arguments[argument_name])
            except KeyError:
                raise ArgumentError('key not found')
            except AttributeError:
                raise DecodeAlgorithmNotFoundError(algo)

        return self.f(self.obj, *k, **kw)

    def decode_base64(self, value):
        """
        Implemente le decodage base64.
        """

        # base64 decoding returns bytestrings
        decoded_value = base64.b64decode(value).decode("utf-8")
        return decoded_value



# -*- coding: utf-8 -*-
"""
class to run all the process
"""

from excalibur.loader import ConfigurationLoader, PluginLoader
from excalibur.check import CheckACL,\
    CheckArguments, CheckRequest, CheckSource
from excalibur.decode import DecodeArguments
from excalibur.exceptions import PluginRunnerError, WrongSignatureError
from importlib import import_module
import base64
import hashlib
from functools import reduce
from excalibur.utils import add_args_then_encode, get_api_keys, ALL_KEYWORD,\
    PLUGIN_NAME_SEPARATOR, SOURCE_SEPARATOR, get_targeted_sources_for_all
from excalibur.conf import Sources


class PluginsRunner(object):

    # 22/04/2014 :checksign defaults to false

    def __init__(self, acl, sources, ressources,
                 plugins_module, check_signature=True, check_ip=True,
                 raw_yaml_content=False):
        self.__raw_yaml_content = raw_yaml_content

        self["acl"] = acl
        self["sources"] = sources
        self["ressources"] = ressources
        self["plugins_module"] = plugins_module
        self["check_signature"] = check_signature
        self["check_ip"] = check_ip

    @property
    def acl(self):
        return self.__acl

    @property
    def ressources(self):
        return self.__ressources

    @property
    def plugins_module(self):
        return self.__plugins_module

    def plugins(self, source, signature, arguments=None, project=None):
        """
        returns allowed plugins
        """
        try:
            if source != ALL_KEYWORD and SOURCE_SEPARATOR not in source:

                return self.sources(signature,
                                    project,
                                    arguments)[source]["plugins"]
            else:
                targeted_sources = get_targeted_sources_for_all(signature,
                                                                self.__sources[project],
                                                                arguments,
                                                                self.__check_signature)

                formatedPluginsList = {}
                for k, v in targeted_sources.items():
                    for clef, valeur in v['plugins'].items():
                        formatedPluginsList[
                            k + PLUGIN_NAME_SEPARATOR + clef] = valeur

                return formatedPluginsList
        except KeyError as k:
            raise PluginRunnerError("no such plugin found")

    def sources(self, signature, project=None, arguments=None):
        """
        Since the sources are either registered at top-level
        in the matching yml file or distibuted by projects
        sources() works as a filter to return either the whole
        yml, or the matching entries.
        """

        if project:
            try:
                return self.__sources[project]["sources"]
            except KeyError as k:
                raise PluginRunnerError("no such source found")
        else:
            return self.__sources

    def __setitem__(self, key, value):
        setattr(self, "_" + self.__class__.__name__ + "__" + key,
                self.resolve(value, key))

    def resolve(self, file, key):
        return ConfigurationLoader(file,
                                   self.__raw_yaml_content,
                                   key=key
                                   ).content if\
            key in ["acl", "sources", "ressources"]\
            else file

    def sources_names(self, project=None):
        """
        Return all sources' names of a project
        """
        if project:
            try:
                return sorted(self.__sources[project]["sources"].keys())
            except KeyError:
                raise PluginRunnerError("no such source found")
        else:
            return sorted(self.__sources.keys())

    def check_all(foo):
        """
        Check all yml
        """

        def checks(self, query):

            module = import_module('excalibur.check')
            check_list = [
                'CheckSource',
                'CheckACL',
                'CheckRequest',
                'CheckArguments'
            ]

            def checker(x):
                checker = getattr(module, x)
                checker(query, self.__ressources,
                        self.sources(query.signature if
                                     query.signature else None,
                                     query.project,
                                     query.arguments if
                                     query.arguments else None),
                        self.__acl,
                        sha1check=self.__check_signature,
                        ipcheck=self.__check_ip)()
            list(map(checker, [method_name for method_name in dir(module) if
                               method_name in check_list]))

            return foo(self, query)

        return checks

    @check_all
    def __call__(self, query):
        data, errors = self.run_plugins(query)
        return data, errors

    def run_plugins(self, query):
        """
        Takes the query as argument and
        browses plugins to execute methods it requires.
        run_plugins is indeed excalibur's core.
        Returns obtained data and errors from all
        launched plugins.
        """

        data, errors = {}, {}
        # Load plugins
        plugin_loader = PluginLoader(self.__plugins_module)

        # Get plugins depending on the sources.yml depth
        plugins = self.plugins(query.source,
                               query.signature,
                               query.arguments,
                               query.project if
                               query.project else None,
                               )
        # Name of the function to run
        f_name = "%s_%s" % (query.ressource, query.method)

        # Actually browse plugins to launch required methods
        # First loop over registered plugins.
        for plugin_name, parameters_sets in plugins.items():
            # Load plugin
            raw_plugin_name = plugin_name
            must_replace_names = False
            # Should become the generic behavior
            if PLUGIN_NAME_SEPARATOR in plugin_name:
                must_replace_names = True
                plugin_name = plugin_name[
                    plugin_name.index(PLUGIN_NAME_SEPARATOR) + 1:]
            plugin = plugin_loader.get_plugin(plugin_name)
            # Then loop over each plugin registered parameters, with
            # an index so that the error can specify which parameter
            # raised the error.
            for parameters_index, parameters in enumerate(parameters_sets):
                # Initialize returned data to None
                plugin_data = None

                if not hasattr(plugin, f_name):
                    continue  # Ressource/method not implemented in plugin
                # Get data
                try:
                    f = getattr(plugin, f_name)
                    plugin_data = f(parameters, query.arguments)
                # Or register exception
                except Exception as e:
                    errors[plugin_name] = {
                        'source': query.source,
                        'ressource': query.ressource,
                        'method': query.method,
                        'arguments': query.arguments,
                        'parameters_index': parameters_index,
                        'error': e.__class__.__name__,
                        'error_message': str(e)
                    }
                # Register data by plugin name
                if plugin_data is not None:
                    if must_replace_names:
                        data[raw_plugin_name] = plugin_data
                    else:
                        data[plugin_name] = plugin_data

        return data, errors


class Query(object):

    """
    Queries are client requests.
    They bear data concerning both the client
    and what he requires from the api.
    """

    def __init__(self, source,
                 remote_ip,
                 ressource,
                 method,
                 request_method,
                 signature=None,
                 project=None,
                 arguments=None):

        self["project"] = project
        self["source"] = source
        self["remote_ip"] = remote_ip
        self["signature"] = signature
        self["arguments"] = arguments if arguments else {}
        self["ressource"] = ressource
        self["method"] = method
        self["request_method"] = request_method

    def __str__(self):
        return "project:%s,source:%s,ip:%s,sign:%s,args:%s,ressource:%s,\
method:%s, request_method:%s" % (self.__project, self.__source,
                                 self.__remote_ip, self.__signature,
                                 self.__arguments, self.__ressource,
                                 self.__method, self.__request_method)

    @property
    def project(self):
        return self.__project

    @property
    def source(self):
        return self.__source

    @property
    def remote_ip(self):
        return self.__remote_ip

    @property
    def signature(self):
        return self.__signature

    @property
    def arguments(self):
        return self.__arguments

    @property
    def ressource(self):
        return self.__ressource

    @property
    def method(self):
        return self.__method

    @property
    def request_method(self):
        return self.__request_method

    def __setitem__(self, key, value):
        setattr(self, "_" + self.__class__.__name__ + "__" + key,
                value)



# -*- coding: utf-8 -*-


class Sources(object):

    """
    """
    structure_type = {'3': 'project', '1': 'flat'}

    def get_level(self):
        """
        """
        def find_plugins(l, i):
            where_to_search = l[0] if type(l) == list else l
            if "plugins" in where_to_search.keys():
                return i
            else:
                i = i + 1
                return find_plugins(where_to_search.values(), i)
        return find_plugins(self.__dict__, 0)

    def __getitem__(self, key):
        """
        """
        ret = None
        try:
            ret = getattr(self, key)
        except AttributeError as ae:
            raise KeyError()
        return ret

    def keys(self):
        """
        """
        return self.__dict__.keys()

    def values(self):
        """
        """
        return self.__dict__.values()


class Ressources(object):

    """
    """

    def __init__(self):
        """
        """

    def __getitem__(self, key):
        """
        """
        ret = None
        try:
            ret = getattr(self, key)
        except AttributeError as ae:
            raise KeyError()
        return ret

    def keys(self):
        """
        """
        return self.__dict__.keys()


class Acl(object):

    """
    """

    def __getitem__(self, key):
        """
        """
        ret = None
        try:
            ret = getattr(self, key)
        except AttributeError as ae:
            raise KeyError()
        return ret



# -*- coding: utf-8 -*-
"""
Cross-classes utils
"""
import hashlib
import traceback
import re
from excalibur.exceptions import WrongSignatureError

ALL_KEYWORD = "all"
PLUGIN_NAME_SEPARATOR = "|"
SOURCE_SEPARATOR = ","


def add_args_then_encode(x, y, arguments):
    """
    sha1checks are made on aggregated Strings. The process is used
    in two spots. When checks regarding the sources.yml file are done
    and in the case where allowed sources cannot be deduced from query
    arguments but must be be obtained by checking apikeys directly in all
    sources.
    """
    def add_args(x, args):
        """
        add arguments to main key before encoding
        """
        for argument in args:
            x += (argument + arguments[argument])
        return x

    def encode(x):
        """
        encode full string
        """

        return hashlib.sha1(x.encode("utf-8")).hexdigest()

    return encode(add_args(x, y))


def get_api_keys(entry, arguments):
    """
    """
    keys = []
    api_keys = []

    if type(entry['apikey']) is list:
        for k in entry['apikey']:
            keys.append(k)
    elif type(entry['apikey']) is str:
        keys.append(entry['apikey'])

    for key in keys:
        api_key = add_args_then_encode(key,
                                       sorted(arguments),
                                       arguments)
        if api_key not in api_keys:
            api_keys.append(api_key)

    return api_keys


def sources_list_or_list(source):
    """
    """
    return list(source.split(SOURCE_SEPARATOR)) if\
        SOURCE_SEPARATOR in source else [source]


def all_sources_or_sources_list_or_list(source, sources):
    """
    """
    return list(sources.keys())\
        if source == ALL_KEYWORD else sources_list_or_list(source)


def is_simple_request(source, sources):
    """
    """
    return source != ALL_KEYWORD and\
        SOURCE_SEPARATOR not in source


def is_simple_request_and_source_not_found(source, sources):
    """
    """
    return is_simple_request(source, sources) and source not in sources.keys()


def ip_found_in_sources(source, sources, request_ip):
    """
    """
    ip_authorized = True

    targeted_sources = []
    if is_simple_request(source, sources):
        targeted_sources = [sources[source].get('ip',[])]
    else:
        targeted_sources = [it["ip"] for
                            it in sources.values() if "ip" in
                            list(it.keys())]
    for ip_list in targeted_sources:
        if not [ip for ip in ip_list if re.match(ip, request_ip)]:
            ip_authorized = False
    return ip_authorized 


def get_api_keys_by_sources(sources, targets):
    """
    """
    def get_keys(x):
        if type(sources[x]["apikey"]) is list:
            return sources[x]["apikey"]
        else:
            return[sources[x]["apikey"]]

    return {target: get_keys(target) for target in targets}


def get_targeted_sources_for_all(signature, data_project,
                                 arguments, check_signature):

    apikey_present = [it["apikey"]
                      for it in list(data_project["sources"].values())
                      if "apikey" in list(it.keys())]

    if apikey_present and check_signature:

        targeted_sources = {}

        for name, value in data_project["sources"].items():

            api_keys = get_api_keys(value, arguments)

            if signature in api_keys:
                targeted_sources[name] = value

        if not targeted_sources:
            raise WrongSignatureError(signature)

        return targeted_sources

    else:
        return data_project["sources"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
from unittest import TestCase, main
from excalibur.decode import DecodeArguments
from excalibur.loader import ConfigurationLoader
from excalibur.check import CheckSource, CheckACL, CheckRequest, \
    CheckArguments, Check
from excalibur.exceptions import SourceNotFoundError, IPNotAuthorizedError,\
    WrongSignatureError, NoACLMatchedError, RessourceNotFoundError,\
    MethodNotFoundError, HTTPMethodError, ArgumentError, \
    ArgumentCheckMethodNotFoundError, ExcaliburError, \
    DecodeAlgorithmNotFoundError
from excalibur.core import PluginsRunner, Query
import base64
from importlib import import_module
from excalibur.utils import ALL_KEYWORD
 
 
class CheckTest(TestCase):
 
    """
    Unit test for check method
    Warning : "Params" of the setUp method need to be valid with
    your yaml files
    """
 
    def setUp(self):
        # Params of the deactivate method for uds
        self.query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
 
        self.query2 = Query(
            source="etab1",
            remote_ip="9.9.9.9",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        encodedzombie = base64.b64encode(b"testzombie1")
        self.query3 = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            arguments={"login": encodedzombie, },
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            ressource="actions",
            method="action1",
            request_method="GET")
        # Files
        self.plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressources.yml",
            "tests.plugins",)
 
    """
    Check Source
    """
 
    def test_check_source(self):
        """ test check sources """
        try:
            CheckSource(
                self.query, None,
                self.plugin_runner.sources(self.query.project), None)()
 
        except:
            self.fail("Error check source")
 
    def test_check_source_not_found(self):
        """ test check sources """
        query1 = Query(
            source="etabnull",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        with self.assertRaises(SourceNotFoundError):
            CheckSource(query1,
                        None,
                        self.plugin_runner.sources(query1.project), None)()
 
    def test_check_source_ip_not_authorized(self):
        """ test check sources """
        with self.assertRaises(IPNotAuthorizedError):
            CheckSource(
                self.query2,
                None,
                self.plugin_runner.sources(self.query2.project), None)()
 
    def test_check_source_ip_with_regexp(self):
        plugin_runner2 = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sourcesipregexp.yml",
            "./tests/data/ressources.yml",
            "tests.plugins")
        try:
            CheckSource(
                self.query2, None,
                plugin_runner2.sources(self.query2.project), None)()
        except:
            self.fail("Error test_check_source_ip_with_regexp")
 
    def test_check_source_signature_error(self):
        """ test check sources """
        query3 = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="ERROR",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
 
        with self.assertRaises(WrongSignatureError):
            CheckSource(
                query3,
                None,
                self.plugin_runner.sources(query3.project),
                None)()
    """
    Check ACL
    """
 
    def test_check_acl(self):
        """ test check acl """
        try:
            CheckACL(self.query, None, None, self.plugin_runner.acl)()
        except Exception as e:
            self.fail("Error check acl")
 
    def test_check_acl_no_matched(self):
        """ test check acl """
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="actionull",
            request_method="GET"
        )
        with self.assertRaises(NoACLMatchedError):
            CheckACL(query, None, None, self.plugin_runner.acl)()
 
        query2 = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET",
            project="keyerror"
        )
        with self.assertRaises(NoACLMatchedError):
            CheckACL(query2, None, None, self.plugin_runner.acl)()
 
        query3 = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="wrongmethod",
            request_method="GET",
            project="project1"
        )
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressources.yml",
            "tests.plugins",
            query3)
 
        with self.assertRaises(NoACLMatchedError):
            CheckACL(query3, None, None, plugin_runner.acl)()
 
    """
    Check Request
    """
 
    def test_check_request(self):
        """ test check request """
        try:
            CheckRequest(
                self.query, self.plugin_runner.ressources, None, None)()
        except:
            self.fail("error check request")
 
    def test_check_request_ressource_not_found(self):
        """ test check request """
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="ressourcenull",
            method="action1",
            request_method="GET"
        )
        with self.assertRaises(RessourceNotFoundError):
            CheckRequest(query, self.plugin_runner.ressources, None, None)()
 
    def test_check_request_method_not_found(self):
        """ test check request """
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="methodnull",
            request_method="GET"
        )
        with self.assertRaises(MethodNotFoundError):
            CheckRequest(query, self.plugin_runner.ressources, None, None)()
 
    def test_check_request_method_not_specified(self):
        """ test check request """
        error = None
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="POST"
        )
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressourcewithoutrequestmethod.yml",
            "tests.plugins",
            query)
        try:
            CheckRequest(query, plugin_runner.ressources, None, None)()
        except Exception as e:
            error = e
        self.assertTrue(not error)
 
    def test_check_request_method_http_error(self):
        """ test check request """
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GETnull"
        )
        with self.assertRaises(HTTPMethodError):
            CheckRequest(query, self.plugin_runner.ressources, None, None)()
 
    def test_check_request_args_error(self):
        """ test check request """
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments="argumentsnull",
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        query2 = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments="argumentsnull",
            ressource="actions",
            method="action2",
            request_method="GET"
        )
        with self.assertRaises(ArgumentError):
            CheckRequest(query, self.plugin_runner.ressources, None, None)()
# Not required anymore, when there is no arguments entry,
# no checks are made
#         plugin_runner = PluginsRunner(
#             "./tests/data/acl.yml",
#             "./tests/data/sources.yml",
#             "./tests/data/ressourceswrongcheck.yml",
#             "tests.plugins",
#             self.query)
#
#         with self.assertRaises(ArgumentError):
#             CheckRequest(query2,plugin_runner.ressources)()
 
    """
    Check arguments
    """
 
    def test_check_arguments(self):
        """ test check arguments """
        try:
#             DecodeArguments(self.query,
#                             self.plugin_runner.ressources)()
            CheckArguments(
                self.query, self.plugin_runner.ressources, None, None)()
 
        except:
            self.fail("error check arguments")
 
    def test_check_arguments_error(self):
        """ test check arguments """
 
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "a", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        with self.assertRaises(ArgumentError):
#             DecodeArguments(self.query,
#                             self.plugin_runner.ressources)()
            CheckArguments(query, self.plugin_runner.ressources, None, None)()
 
        query2 = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "a", },
            ressource="wrong ressource",
            method="action1",
            request_method="GET"
        )
        with self.assertRaises(ArgumentError):
            CheckArguments(query2, self.plugin_runner.ressources, None, None)()
 
    def test_check_value_in(self):
        check_arguments = CheckArguments(self.query,
                                         self.plugin_runner.ressources,
                                         None,
                                         None)
        r = check_arguments.check_value_in("yes", ["yes", "no"])
        self.assertTrue(r)
 
        r = check_arguments.check_value_in("not", ["yes", "no"])
        self.assertFalse(r)
 
    def test_check_matches_re(self):
        check_arguments = CheckArguments(self.query,
                                         self.plugin_runner.ressources,
                                         None,
                                         None)
        regex = "^[a-zA-Z0-9_.-]+@[a-zA-Z0-9_.-]+\.[a-zA-Z]{2,4}$"
        r = check_arguments.check_matches_re("noreply@noreply.com",
                                             regex)
        self.assertTrue(r)
        regex2 = "^[a-zA-Z0-9_.-]+@[a-zA-Z0-9_.-]+\.[a-zA-Z]{2,4}$"
        r = check_arguments.check_matches_re("wrongmail",
                                             regex2)
        self.assertFalse(r)
 
    def test_check_not_exist(self):
 
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressourceswrongcheck.yml",
            "tests.plugins", )
 
        with self.assertRaises(ArgumentCheckMethodNotFoundError):
            CheckArguments(self.query, plugin_runner.ressources, None, None)()
 
    """
    Check all
    """
 
    def test_check_all(self):
        try:
            self.plugin_runner(self.query)
        except:
            self.fail("error check all")
 
    def test_signature_not_required(self):
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        error = None
        try:
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressources.yml",
                "tests.plugins",
                check_signature=False)
        except Exception as e:
            error = "error"
        self.assertTrue(not error)
 
    def test_signature_explicitely_required(self):
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            arguments={"login": "testzombie1", },
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        error = None
        try:
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressources.yml",
                "tests.plugins",
                check_signature=True)
        except Exception as e:
            error = "error"
        self.assertTrue(not error)
 
    def test_encode_base64(self):
 
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressourceswithencodingrequired.yml",
            "tests.plugins")
        CheckArguments(self.query3, plugin_runner.ressources, None, None)()
        self.assertEqual(self.query3.arguments["login"], "testzombie1")
 
    def test_encode_base64_attribute_error(self):
        with self.assertRaises(DecodeAlgorithmNotFoundError):
 
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithundecodableencoding.yml",
                "tests.plugins")
            CheckArguments(self.query3, plugin_runner.ressources, None, None)()
 
    def test_encode_base64_key_error(self):
        try:
            encodedzombie = base64.b64encode(b"testzombie1")
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                arguments={"login": encodedzombie, },
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                ressource="actions",
                method="action1",
                request_method="GET")
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data//noargressources.yml",
                "tests.plugins")
            CheckArguments(query, plugin_runner.ressources, None, None)()
 
        except Exception as e:
            self.assertTrue(isinstance(e, ExcaliburError))
 
    def test_all_error_raised(self):
        module = import_module('excalibur.exceptions')
        message = "message sublime"
 
        def str_caller(e):
 
            exception_class = getattr(module, e)
            exception_instance = exception_class(message)
            errorList = ['ExcaliburInternalError',
                         'ExcaliburClientError',
                         'ExcaliburError']
            if exception_instance.__class__.__name__ not in errorList:
                exceptionName = exception_instance.__class__.__name__
                self.assertEqual('%s : %s' % (exceptionName, message),
                                 exception_instance.__str__())
            else:
                self.assertEqual(message, exception_instance.__str__())
 
        [str_caller(error)
         for error in dir(module) if not error.startswith('_')]
 
    def test_key_errors(self):
        """
        KeyErrors should not be explicitly raised,
        they should instead be caught, and raise
        ExcaliburErrors.
        """
 
        """
        Generate queries with wrong values and launches checks
        """
        def query_generator(pos):
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                arguments={"login": "testzombie1", },
                ressource="actions",
                method="action1",
                request_method="GET"
            )
            setattr(query, pos, "wrong")
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml", "./tests/data/sources.yml",
                "./tests/data/ressources.yml",
                "tests.plugins",
                query)
 
            # with self.assertRaises(ExcaliburError):
            try:
                CheckSource(query, plugin_runner.ressources, plugin_runner.sources(query.signature if
                                     query.signature else None,
                                     query.project,
                                     query.arguments if
                                     query.arguments else None), None)()
                CheckRequest(query, plugin_runner.ressources, None, None,)()
                CheckArguments(query, plugin_runner.ressources, None, None,)()
                CheckACL(query, None, None, plugin_runner.acl)()
 
            except Exception as e:
                self.assertTrue(isinstance(e, ExcaliburError))
 
            return query
 
        query_generator_by_arg = lambda pos: query_generator(pos)
 
        [query_generator_by_arg(attr) for attr in dir(self.query)
         if attr.startswith("_Query") and attr not in ["_Query__remote_ip"]]
 
    def test_check_not_implemented(self):
        c = Check()
        self.assertRaises(NotImplementedError, c.check)
 
    def test_multiple_api_keys(self):
        error = "no_error_yet"
 
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sourceswithmultipleapikeys.yml",
            "./tests/data/ressources.yml",
            "tests.plugins"
        )
        try:
            plugin_runner(self.query)
        except Exception as e:
            error = "error"
        self.assertTrue(error == "no_error_yet")
     
    def test_multiple_api_keys_project_depth_and_etab_set_to_all(self):
        error = "no_error_yet"
        plugin_runner = PluginsRunner(
            "./tests/data/acl_projects.yml",
            "./tests/data/source_project_multiple_api_keys.yml",
            "./tests/data/ressources.yml",
            "tests.plugins"
        )
        query = Query(
            source=ALL_KEYWORD,
            remote_ip="127.0.0.1",
            signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
            arguments={"login": "testzombie1", },
            ressource="actions",
            method="action1",
            request_method="GET",
            project="project1"
        )
        try:
            result = plugin_runner(query)
        except Exception as e:
            error = "error"
        self.assertTrue(error == "no_error_yet")
 
    def test_no_apikey_specified(self):
        error = "no_error_yet"
 
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sourceswithnoapikey.yml",
            "./tests/data/ressources.yml",
            "tests.plugins"
        )
        try:
            plugin_runner(self.query)
        except Exception as e:
            error = "error"
        self.assertTrue(error == "no_error_yet")
 
    def test_no_ip_required(self):
        error = "no_error_yet"
 
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sourceswithnoip.yml",
            "./tests/data/ressources.yml",
            "tests.plugins"
        )
        try:
            plugin_runner(self.query)
        except Exception as e:
            error = "error"
        self.assertTrue(error == "no_error_yet")
 
    def test_no_ip_specified(self):
        error = "no_error_yet"
 
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sourceswithnoip.yml",
            "./tests/data/ressources.yml",
            "tests.plugins"
        )
        try:
            plugin_runner(self.query)
        except Exception as e:
            error = "error"
        self.assertTrue(error == "no_error_yet")
 
    def test_ressources_arguments_not_present(self):
        error = None
        try:
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithnoarguments.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(self.query)
        except Exception as e:
            error = e
        self.assertTrue(not error)
 
    def test_ressources_arguments_empty_and_one_argument_provided(self):
        with self.assertRaises(ArgumentError):
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                arguments={"login": "testzombie1", "uselessarg": "useless"},
                ressource="actions",
                method="action1",
                request_method="GET"
            )
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithemptyargs.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(query)
 
    def test_optional_ressource_argument(self):
        error = None
        try:
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithoptionalargs.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(self.query)
        except Exception as e:
            error = e
        self.assertTrue(not error)
 
    def test_optional_ressource_argument_and_matching_request_arg(self):
        error = None
        try:
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithoptionalargs.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(self.query)
        except Exception as e:
            error = e
        self.assertTrue(not error)
 
    def test_optional_ressourcearg_matching_reqarg_and_exceeding_arg(self):
        with self.assertRaises(ArgumentError):
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                arguments={
                    "login": "testzombie1", "uselessarg": "useless",
                    "exceedingarg": "exceeding"},
                ressource="actions",
                method="action1",
                request_method="GET"
            )
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithoptionalargs.yml",
                "tests.plugins",
                check_signature=False,
            )
            plugin_runner(query)
 
    def test_ressources_arguments_empty_and_no_argument_provided(self):
        error = "no_error_yet"
        query = Query(
            source="etab1",
            remote_ip="127.0.0.1",
            signature="azertyuiop12345",
            arguments={},
            ressource="actions",
            method="action1",
            request_method="GET"
        )
        plugin_runner = PluginsRunner(
            "./tests/data/acl.yml",
            "./tests/data/sources.yml",
            "./tests/data/ressourceswithemptyargs.yml",
            "tests.plugins",
            check_signature=False)
        plugin_runner(query)
        self.assertTrue(error == "no_error_yet")
 
    def test_signature_not_in_generated_signatures(self):
        with self.assertRaises(WrongSignatureError):
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="azertyuiop12345",
                arguments={"login": "testzombie1", },
                ressource="actions",
                method="action1",
                request_method="GET"
            )
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sourceswithmultipleapikeys.yml",
                "./tests/data/ressources.yml",
                "tests.plugins"
            )
            plugin_runner(query)
             
 
    def test_checks_with_optional_args_and_bad_argument_value(self):
        with self.assertRaises(ArgumentError):
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                arguments={"login": "testzombie1", "uselessarg": "a"},
                ressource="actions",
                method="action2",
                request_method="GET"
            )
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithoptionalargs.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(query)
 
    def test_checks_with_optional_args_and_no_arg(self):
        error = None
        try:
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
                arguments={"login": "testzombie1"},
                ressource="actions",
                method="action2",
                request_method="GET"
            )
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithoptionalargs.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(query)
        except Exception as e:
            error = e
        self.assertTrue(error is None)
 
    def test_missing_args(self):
        with self.assertRaises(ArgumentError):
            query = Query(
                source="etab1",
                remote_ip="127.0.0.1",
                signature="c08b3ff9dff7c5f08a1abdfabfbd24279e82dd10",
 
                ressource="actions",
                method="action2",
                request_method="GET"
            )
            plugin_runner = PluginsRunner(
                "./tests/data/acl.yml",
                "./tests/data/sources.yml",
                "./tests/data/ressourceswithoptionalargs.yml",
                "tests.plugins",
                check_signature=False)
            plugin_runner(query)


if __name__ == '__main__':
    main()
